<?php

class Infrastructurerepublic { 
    private $actingUserID;
    private $publicAPIKey;
    private $privateAPIKey;
    private $server;
    
    private $queryString;
    private $hash;
    private $response;

    public function __construct($actingUserID, $publicAPIKey, $privateAPIKey, $server) {
        $this->actingUserID = $actingUserID;
        $this->publicAPIKey = $publicAPIKey;
        $this->privateAPIKey = $privateAPIKey;
        $this->server = $server;        
        
        $this->queryString = "actingUserID=$actingUserID&auth=$publicAPIKey";
        $this->hash = hash('sha256', $privateAPIKey . $this->queryString);
    }
    
    public function get($action, $data = array(), $params=array()) {
        return json_decode($this->request("GET", $action, $data, $params), true);
    }

    public function getApiParams($params = array()) {
		$qStr = $this->queryString;
		$append = '';
		foreach($params as $key=>$val){
			$append.= $key."=".urlencode($val)."&";
		}
        $qStr = $append . $qStr;
        $hash = hash('sha256', $this->privateAPIKey . $qStr);
		$qStr = $qStr . "&hash=".$hash;
		return $qStr;
    }
    
    public function post($action, $data = array()) {
        return $this->request("POST", $action, $data);
    }
    
    public function put($action, $data = array()) {
        return $this->request("PUT", $action, $data);
    }
    
    public function delete($action, $data = array()) {
        return $this->request("DELETE", $action, $data);
    }
    
    private function request($method, $action, $data, $params=array()) {
        $header = array('Content-Type: application/json', 'Accept:application/json');
		$params = $this->getApiParams($params);
        $url = "http://$this->server/$action?$params";
        $data_json = json_encode($data);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        if($method != "DELETE") 
            curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json','Content-Length: ' . strlen($data_json)));
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_POSTFIELDS,$data_json);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response  = curl_exec($ch);
		$retry = 0;
		while(curl_errno($ch) == 28 && $retry < 3){
			$response = curl_exec($ch);
			$retry++;
		}
        curl_close($ch);
        
        logModuleCall('Infrastructurerepublic', $method . ' ' . $action, array('method'=>$method,'action'=>$action, 'data'=>$data, 'params'=>$params), $response);  

        $this->response = json_decode($response, true);
        return $response;
    }
    

    public function getPromoStatus($domainId, $domainName, $period, $userId, $actionType) {
        $res = mysql_safequery("SELECT * FROM `tblorders` o JOIN `tbldomains` d ON o.id = d.orderid WHERE d.id =?",array($domainId));
		$row = mysql_fetch_assoc($res);
		if(isset($row['promocode']) && trim($row['promocode'])!=''){
			$promoCode = trim($row['promocode']);
			$action = "registrar/user/$userId/period/$period/trans-type/$actionType/domain/$domainName";
			$record = $this->get($action, array(), array("promoCode"=>$promoCode));			
			$promoAvailable = false;
			$promoMessage = '';
			if(isset($record['response'])){
				$record = $record['response'];
				if(isset($record['PromoStatus']) && $record['PromoStatus'] == "true"){
					$promoAvailable = true;
					$promoMessage = '';
				}else{
					$promoAvailable = false;
					$promoMessage = (isset($record['ErrorMessage']) && $record['ErrorMessage']!='') ? $record['ErrorMessage'] : "Failed to apply Promotion Code.";
				}
			}else{
				$promoMessage = (isset($record['message']) && $record['message']!='') ? $record['message'] : "Failed to apply Promotion Code.";
			}
			return array(
				"promoCode" => $promoCode,	
				"promoApplicable" => true,	
				"promoAvailable" => $promoAvailable,	
				"promoMessage" => $promoMessage,	
			);
		}
		else
		{
			$promoAvailable = false;
			$promoMessage = '';
			return array(
				"promoCode" => '',	
				"promoApplicable" => false,	
				"promoAvailable" => false,	
				"promoMessage" => '',	
			);
		}
    }

    public function addContact($tld,$details) {
        return $this->post('registrar/contact/'.$tld,$details);
    }
    
    public function getResponse() {
        return $this->response;
    }
    
    public function getResultCode() {
        return $this->response['resultCode'];
    }
    
    public function getError() {
        return $this->response['message'];
    }
    
    public function checkAvailability($domain) {
        return $this->get('registrar/domain/check/'.$domain);
    }
    
    public function getInvoiceData($id) {
        return $this->get('invoice/' . $id);
    }
    
    public function createInvoice($params,$type='registration') {
        
        $data = array(
            'soldBy' => $params['soldby'],
            'soldTo' => $params['soldto'],
            'transactionType' => $type,
            "period" => $params['regperiod'],
            "name" => $params['name'],
            "nameservers" => $params['nameservers'],
            "nameserversType" => "specify-ns",
            "registrant"    =>  $params['contacts']['registrant'],
            "contacts" => array(
                "admin" => $params['contacts']['admin'],
                "billing" => $params['contacts']['billing'],
                "tech" => $params['contacts']['tech']
            )
        );
        
        if($params['magicfolderid']){
            $data['metadata'] = array(
                "magicFolderID" => $params['magicfolderid'],
            );
        }
		if($params['authcode']){
            $data['authcode'] = $params['authcode'];
        }
        if($params['promoCode']){
            $data['promoCode'] = $params['promoCode'];
        }
        return $this->post('invoice/orderwithcheck/domain/',$data);
    }

	
    public function editOrder($id,$params) {
        return $this->put('order/'.$id,$params);
    }

    public function chargeInvoice($id,$period, $soldto) { 
      return $this->put('invoice/charge/'.$id,array(
            "period" => $period,
            "metadata" => array(
                        "creditNoteUserID" => $soldto,
                        ),            
      ));
    }
    public function orderRenew($orderid,$params) { 
		$data = array("period" => $params['period']);
		if(isset($params['promoCode'])){
			$data['promoCode'] = $params['promoCode'];
		}
      return $this->put('order/renewwithcheck/'.$orderid,$data);
    }
    
    public function editContact($tld,$contactid,$details) {
        
        return $this->put('registrar/contact/'.$tld.'/'.$contactid, $details);
    }
    
    public function getDomainInfo($domain) {
        return $this->get('registrar/domain/'.$domain);
    }
    public function getDomainContactDetails($tld,$contactid) {
        return $this->get('registrar/contact/'.$tld.'/'.$contactid);
    }
    public function setNameServers($tab) {
        return $this->put('registrar/domain/'.$tab['name'], $tab);
    }
    
    public function addChildNameServer($tld,$name,$addresses,$soldto) {
        return $this->post('registrar/host/'.$tld,array(
            "name"          =>  $name,
            "ipAddresses"   =>  $addresses,
            "metadata"      =>  array("userID" => $soldto),
        ));
    }
    
    public function editChildNameServer($tld,$name,$edit_addresses,$remove_addresses,$soldto) {
    
        $params = array(
            "name"               =>  $name,
            "metadata"           =>  array("userID" => $soldto),
        );
        $params['ipAddresses'] = $edit_addresses;
        $params['removeIpAddresses'] = $remove_addresses;

        return $this->put('registrar/host/'.$tld,$params);
    }
    

    public function getPromoIfApplicable($tld,$orderid,$customerid,$actiontype,$soldto) {
    	
    	$res = mysql_safequery("SELECT * FROM `tblorders` o JOIN `tbldomains` d ON o.id = d.orderid WHERE d.id =?",array($params['domainid']));
		$row = mysql_fetch_assoc($res);
		$promoCode = isset($row['promocode'])?$row['promocode']:'';
		
        $params = array(
            "name"               =>  $name,
            "metadata"           =>  array("userID" => $soldto),
        );
        $params['ipAddresses'] = $edit_addresses;
        $params['removeIpAddresses'] = $remove_addresses;

        return $this->put('registrar/host/'.$tld,$params);
    }
    
    public function addCustomer($params, $randpwd) { //?

        $ipv6 = '0000:0000:0000:0000:0000:0000:'.$_SERVER['REMOTE_ADDR'];
        $arrIP = explode(':', $ipv6);
        if( preg_match('/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/', $arrIP[count($arrIP)-1]) ) {
          $ip4parts = explode('.', $arrIP[count($arrIP)-1]);
          $ip6trans = sprintf("%02x%02x:%02x%02x", $ip4parts[0], $ip4parts[1], $ip4parts[2], $ip4parts[3]);
          $arrIP[count($arrIP)-1] = $ip6trans;

          $ipv6 = implode(':', $arrIP);
        }
 
        $cur = mysql_fetch_array(full_query("SELECT code FROM tblcurrencies WHERE id=".$params['original']['currency']));
        $cur = $cur['code'];
        $data = array(
            "type" => "customer",
            "email" => $params['email'], 
            "password" => $randpwd, 
            "parentID" => $params['soldby'],
            "autoSignup" => 1,
            "buyingCurrency" => $cur, 
            "allow" => array(
                ""
            ),
            "allowIPV" => array(
                array(
                    "ipv4address" => $_SERVER['REMOTE_ADDR']
                )
            ),
            "socialPages" => array(),
            "socialInfo" => array(),
            "userInfo" => array(
                "dateOfBirth" => "1970-01-01"
            ),
            "contactInfo" => array(
                "postalInfo" => array(
                    "firstName" => $params['firstname'], 
                    "lastName" => $params['lastname'], 
                    "title" => "Mr.",
                    "organization" => $params['companyname'], 
                    "address" => array(
                        "street" => array(
                            $params['original']['address1'], 
                            $params['original']['address2'], 
                            ""
                        ),
                        "city" => $params['original']['city'], 
                        "postalCode" => $params['original']['postcode'], 
                        "state" => $params['original']['fullstate'], 
                        "countryCode" => $params['original']['countrycode'], 
                    ),
                ),
                "voice" => array(
                    "number" => $params['original']['phonenumberformatted'], 
                    "countrycode" => $params['original']['countrycode'],
                    "ext" => "",
                ),
                "callInPin" => "1234"
            ),
            "preferences" => array(
                "phoneNoType" => "work"
            ),
            "head" => array(
                "userIpAddress" => $ipv6 
            )
        );
        
        return $this->post('user/whmcs',$data);
    }
    
    public function activateCustomer($uid) { 
        return $this->put('user/'.$uid,array(
            "status" => 'active',));    
    }
  
    public function pwdGen($length = 9){
        $chars = array(
            'abcdefghijklmnopqrstuvwxyz', 
            'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
            '0123456789',
            '!@#$%&*?'
        );
        $pwd = '';
        $password = '';
        foreach($chars as $char){
            $password .= $char[array_rand(str_split($char))];
            $pwd .= $char;
        }
        
        $pwd = str_split($pwd);
        for($i = 0; $i < $length - count($chars); $i++)
            $password .= $pwd[array_rand($pwd)];

        return str_shuffle($password);
    }
}